
#include <90usb1287.h>
#include <usb_driver.h>
#include <spi.h>
#include <interface.h>
#include <stdio.h>
#include <delay.h>
#include <usb_drv.h>


//#define API_JMP_BOOTLOADER (*((const void(flash*)(void)) 0xF800 ))


extern unsigned char Command;
extern unsigned int  Param;
extern unsigned char BufCnt;
extern unsigned char Plot;
extern unsigned char Step;
eeprom unsigned char skip_boot @0x00;
eeprom unsigned int  deviceID  @0x01;
extern char LineCoding[];
unsigned char Process;
unsigned char PlotTime;


extern unsigned char RunReg;
extern unsigned char adc_data[4];
extern unsigned int  TimeMin;
extern unsigned char TimeSec;
extern unsigned int  Ref_P;  //Pressure reference
extern unsigned int  Ref_T;  //Temperature reference
extern unsigned int Ctrl_A;
extern unsigned int Ctrl_B;

void runCom(void){
        
        switch(Command){
                  
        case(NO_COM):
            Command = 0;
            break;
        
        case(INF):
            printf("university\r\n");
            printf("department\r\n");
            printf("Firmware Version \r\n");
            printf("Compiled: %p %p\r\n",__DATE__,__TIME__);
            Command = 0;
            break;
        
        case(RESET):        
            UDCON |= 0x01;      //Disable USB
            #asm("CLI");        //Disable global interrupt
            if(skip_boot == 0x69) skip_boot = 0xFF;
            MCUSR = 0x00;
            #asm("WDR");
            WDTCSR=0x1B;
            WDTCSR=0x0B;
            while(1);           //Wait for reset
            Command = 0;
            break;
                                           
        case(GOTO_BOOT):            
            #asm("CLI");        //Disable global interrupt
            skip_boot = 0x69;
            #asm("WDR");
            WDTCSR=0x1B;
            WDTCSR=0x0B;
            while(1);           //Wait for reset
            Command = 0;
            break;
               
        case(SETID):       
            while(EECR & 0x02);
            deviceID = Param;
            #asm("cli")     //Disable Global Interrupt
            EECR |= 0x04;
            EECR |= 0x02;
            //Re-enumerate 
            UDCON |= 0x01;  //Set detach bit
            delay_ms(10);
            UDCON &=~0x01;  //Clear detach bit (attach)
            #asm("sei")     //Enable Global Interrupt
            Command = 0;
            break;
            
        case(USB):
            printUSB();
            Command = 0;  
        
        default:
            break;   
        }
}



void main(void){
    
    if(skip_boot == 0x69 || (MCUSR & 0x0A)){    //Check if "boot" initiated prior to reset
        skip_boot = 0xFF;     //Reset boot-check value before continuing
        MCUSR = 0;
        #asm("JMP 0xF000;");  //Proceed to boot-loader-section
    }     

    // Crystal Oscillator division factor: 1
    #pragma optsize-
    CLKPR=0x80;
    CLKPR=0x00;
    #ifdef _OPTIMIZE_SIZE_
    #pragma optsize+
    #endif

    // Input/Output Ports initialization
    PORTA=0xFF; // No "Chip Select"
    DDRA=0xFF;  // All out
    PORTB=0x77; // Gain ctrl + MISO pull-up + No "Chip select"
    //PORTB=0x17; // No Default: No LED's
    DDRB=0xF7;  // 
    PORTC=0xBC;
    DDRC=0xFF;  // 

    PORTD = 0x51; // Leds
    DDRD = 0xF8;  // (UART)
    PORTE = 0x00;
    DDRE = 0x00;
    PORTF = 0x00;
    DDRF = 0x00;

    //PORTB=0x01;
    //DDRB=0x07;


    /*****************************************
        T I M E R  0  -  7813Hz @ 8MHz / 15 625Hz @ 16MHz --> 64us
        Controls:   - Variable scan delay  
    ******************************************/
    TCCR0A=0x00;
    TCCR0B=0x00;  //0x05;
    TCNT0=0x00;
    OCR0A=0x00;
    OCR0B=0x00;


    /*****************************************
        T I M E R  1 - 250KHz -> 10Hz @ 8MHz
        Controls:   - Realtime Clock 
                   
    ******************************************/
    TCCR1A=0x00;
    TCCR1B=0x0B;
    TCNT1H=0x00;
    TCNT1L=0x00;
    ICR1H=0x00;
    ICR1L=0x00;
    OCR1AH=0x61;
    OCR1AL=0xA8;
    OCR1BH=0x00;
    OCR1BL=0x00;
    OCR1CH=0x00;
    OCR1CL=0x00;


    /*****************************************
        T I M E R  2 - Stopped
    ******************************************/
    ASSR=0x00;
    TCCR2A=0x00;
    TCCR2B=0x00;
    TCNT2=0x00;
    OCR2A=0x00;
    OCR2B=0x00;


    // External Interrupt(s) initialization                
    //EICRA=0x02;
    EICRA=0x01; //Any change in level
    EICRB=0x00;
    EIMSK=0x01;
    EIFR=0x01;

    // PCINT0 interrupt: Off
    // PCINT1 interrupt: Off
    // PCINT2 interrupt: Off
    // PCINT3 interrupt: Off
    // PCINT4 interrupt: Off
    // PCINT5 interrupt: Off
    // PCINT6 interrupt: Off
    // PCINT7 interrupt: Off
    PCMSK0=0x00;
    PCICR=0x00;

    // Timer/Counter 0 Interrupt(s) initialization
    TIMSK0=0x02;
    // Timer/Counter 1 Interrupt(s) initialization
    TIMSK1=0x02; //Compare A MATCH Interrupt
    // Timer/Counter 2 Interrupt(s) initialization
    TIMSK2=0x00;

    // USART1 initialization
    // Communication Parameters: 8 Data, 1 Stop, No Parity
    // USART1 Receiver: On
    // USART1 Transmitter: On
    // USART1 Mode: Asynchronous
    // USART1 Baud Rate: 38400
    UCSR1A=0x00;
    UCSR1B=0x98;
    UCSR1C=0x06;
    UBRR1H=0x00;
    UBRR1L=0x0C;


    // spi initialization
    // spi Type: Master
    // spi Clock Phase: Cycle Half
    // spi Clock Polarity: High
    // spi Data Order: MSB First

    /*
    SPI2X SPR1 SPR0 SCK Frequency
    0   0   0   fosc/4
    0   0   1   fosc/16
    0   1   0   fosc/64
    0   1   1   fosc/128
    1   0   0   fosc/2
    1   0   1   fosc/8
    1   1   0   fosc/32
    1   1   1   fosc/64
    */

    EECR = 0x00;  //EEPROM

    SPSR=0x00;          // "Single speed" on SPI

    
    initLineCoding();
    initUSB();
    init_EEPROM();

    SPSR=0x01;          // "Double speed" on SPI

    Command = INF;
    Process = 0;
    PlotTime = 0;
         
    // Global enable interrupts
    #asm("SEI")

    while (1){
          
        // Process 0: Comunication
        if(BufCnt)    getcom();         // If unread material 
        if(Command)   runCom();         // If new cmd       

                    
        // Process 1: StateMachine 
        StateMachine();
        
        // Process 2: Regulator (controlled by realtime clock)
        if(RunReg){
            RunReg--;
            P_Regulator();
            T_Regulator();
             
        }
           
        // Process 3: Plotting (controlled by realtime clock)
        if(PlotTime){
            if(PlotTime & 0x20){
                printf("$V;%u;%u;%u;%u;\r\n",adc_data[0],adc_data[1],adc_data[2],adc_data[3]);
                PlotTime &=~0x20;
            }
                
            if(PlotTime & 0x40){
                printf("$R;%u;%u;%u;%u;%u;%u;%u;\r\n",TimeMin,TimeSec,(unsigned int)Ref_P,69,(unsigned int)Ref_T,Ctrl_A,Ctrl_B);
                PlotTime &=~0x40;
            }
        }
        
              
    }
}
