
#include <90usb1287.h>
#include <stdio.h>
#include <descriptors.h>
#include <interface.h>
#include <string.h>

#define usbTXbufSIZE 192

void DeviceRequest(void);
void initUSB(void);
void USBsend(void);

unsigned char usbTXbuffer[usbTXbufSIZE];
unsigned char UsbTxCnt;
unsigned char UsbRdCnt;
unsigned char UsbBufCnt;

char LineCoding[7];  // CDC-class specific

/*********************************************
   USB endpoint interrupt service routine
**********************************************/ 
interrupt [USB_ENDPOINT_PIPE] void usb_endpoint_isr(void){
            
    if(UEINT & 1){                  //Check for INT on EP0 
        UENUM = 0;                  //Select EP0 
        if (UEINTX & 0x08){         //Check if it's a setup packet (RXSTPI) 
            DeviceRequest();        //Call a function
        }else if(UEINTX & 0x04){    //Check if it's a Received data packet (RXOUTI)
            UEINTX &=~0x04;         //Clear
            if(UEBCLX){
//                printf("OUT data: ");
//                while(UEBCLX) printf("0x%X,",UEDATX);
//                printf("\r\n");
            } 
        }else if(UEINTX & 0x40){    //If other (exception)
                UEINTX &= 0x00;     
        } 
    
    /**********************************
     USB CDC DEVICE
    ********************************/
    
    }else if (UEINT & 2){       //Check for INT on EP1 (BULK_IN -> To PC) 
        UENUM = 1;              //Select EP1
     
        if (UEINTX & 0x01){     //Check for Data (TXINI)
            UEINTX &=~0x01;     //Clear interrupt (TXINI)
            USBsend();
            UEINTX &=~0x80;     //Clear FIFOCON (Make bank ready)
        }
    
    }else if (UEINT & 4){       //Check for INT on EP2 (BULK_OUT -> From PC)
        UENUM = 2;              //Select EP2
        if (UEINTX & 0x04){     //Check for Data (RXINI)
            UEINTX &=~0x04;     //Clear interrupt (RXOUTI)
            catchString();      //Append data
            UEINTX &=~0x80;     //Clear FIFOCON (Make bank ready)
        }
             
    }else if (UEINT & 8){       //Check for INT on EP3 (INT IN -> To PC)
        UENUM = 3;              //Select EP3 
        if (UEINTX & 0x01){     //Check for Data (TXINI)   
            UEINTX &=~0x01;     //Clear interrupt (TXINI)
            // Do something....
            UEINTX &=~0x80;     //Clear FIFOCON (Make bank ready)
        }                     
    }
    //#asm("WDR"); // Reset watch-dog timer if activity

} 


/*********************************************
   USB general interrupt service routine
**********************************************/
interrupt [USB_GENERAL] void usb_general_isr(void){
   
   //unsigned char i;

   if (USBINT & 1){ 
        initUSB();
        USBINT&=~0x01;  // Clear flag
        return;           
   }   
   
   if (UDINT & 0x10){   
      // Wakeup INT
      UDIEN   = 0x08;   // End of Reset INT enable 


      PLLCSR  = 0x14;   // PLL for 16MHz
      PLLCSR |= 0x02;   // PLL enable
   
//      PLLCSR  = 0x0C;   // PLL prescaler for 8Mhz
//      PLLCSR |= 0x02;   // PLL enable
   
               
      while(!(PLLCSR & 0x01));
      
      
      
      USBCON  = 0x90;   // Clk enable
      UDINT  &=~0x10;   // Reset INT flag
      
   }else if (UDINT & 0x08){
      // End of Reset INT
      UENUM   = 0;      // Enpoint 0
      UECONX  = 0x01;   // EP enable
      UECFG0X = 0x00;   // Typ: Control  
      UECFG1X = 0x30;   // Size: 64, 1 Bank
      UECFG1X = 0x32;   // Allocate
      UDINT  &=~0x08;   // Reset INT flag
      UEIENX  = 0x0C;   // Recieve Setup & RXOUTI INT enable
      UEIENX |= 0x40;   // Enable NAK IN (essential)
      LED_BLUE = 1;          
   }else{               
       
      printf("Error");
   } 
}

void ConfigEndpoint(flash char *ptr){

    UENUM = ptr[2] & 0x0F;      // Enpoint no
    UECONX   = 0x01;            // EP enable
    
    UECFG0X  = (ptr[3]<< 6);    // Type: Control, Interrupt, Bulk, Iso
    UECFG0X |= (ptr[2]>>7);     // Direction IN/OUT
    
    // Find EP Size
    if(ptr[4] == 8)          UECFG1X = 0x00;
    else if(ptr[4] == 16)    UECFG1X = 0x10;
    else if(ptr[4] == 32)    UECFG1X = 0x20;
    else if(ptr[4] == 64)    UECFG1X = 0x30;
    else if(ptr[4] == 128)   UECFG1X = 0x40;
    else if(ptr[5] == 1)     UECFG1X = 0x50;
    else if(ptr[5] == 2)     UECFG1X = 0x60;
    else                     UECFG1X = 0x00;
    
    if(ptr[2] & 0x80)        UECFG1X |= 0x04; //Two banks
    //if(ptr[2] == 0x81)        UECFG1X |= 0x04; //Two banks
    //if(ptr[2] == 0x02)        UECFG1X |= 0x04; //Two banks
    
    UECFG1X |= 0x02;   // Allocate
                 
    if(!(UESTA0X & 0x80)){
        printf("&EP%d error\r\n",UENUM);
        return;
    }
                 
    if(ptr[2] & 0x80)  UEIENX  = 0x01;   // Transmitter ready Interrupt Enable    
    else               UEIENX  = 0x04;   // Received OUT data Interrupt Enable 
    
    // Additional interrups
    //UEIENX |= 0x1F; //NAOUTE, RXSTPE, RXOUTE STALLEDE TXINE
          
}

void ResetEndpoint(char n){
     
     UERST |= (0x01 << n);
     #asm("NOP");
     UERST  =  0x00;
}

#define _ALTERNATE_PUTCHAR_
void putchar(unsigned char c){
  
    if(UsbBufCnt){     
        while(UsbBufCnt  >= sizeof(usbTXbuffer)) if(!(UDADDR & 0x80)) return;  // Wait until ready (if adressed)....
        if(UsbTxCnt >= sizeof(usbTXbuffer)) UsbTxCnt = 0;
        usbTXbuffer[UsbTxCnt++] = c;
        UsbBufCnt++;
    }else{
        #asm("cli");
        UENUM = 1;  //Choose EP1 (Tx, double bank)
        if(UEINTX & 0x20){
            UEDATX = c;
            #asm("sei");
        }else{
            #asm("sei");
            if(UsbTxCnt >= sizeof(usbTXbuffer)) UsbTxCnt = 0;
            usbTXbuffer[UsbTxCnt++] = c;
            UsbBufCnt++;
        }
    }
}


void USBsend(void){

    while((UEINTX & 0x20) && UsbBufCnt){       
        UsbBufCnt--;    // Decrement buf-counter
        if(UsbRdCnt >= sizeof(usbTXbuffer)) UsbRdCnt = 0;
        UEDATX = usbTXbuffer[UsbRdCnt++];
    }    
}

void SendDescriptor(flash char *ptr, char j){

    char i;
    i = 0;                  //Start
    if(j == 0) j = ptr[0];  //No of bytes
    
    while(i<j){
        UEDATX = ptr[i++];             // Load data to buffer
        if(UEBCLX == 0x40 || i == j){  // If buffer full or No more data
            UEINTX &= 0xFE;            // Send data
            while(!(UEINTX & 0x01));   // Wait for handshake
        }
    }
}

extern eeprom unsigned int deviceID;


/*****************************************
      D E V I C E  R E Q U E S T S
******************************************/
void DeviceRequest(void){
    unsigned char a, Data[10];
    
    // Read Request
    for (a=0;a<8;a++) Data[a] = UEDATX;
    UEINTX &= 0xF7;   // Clear INT flag
     
    /********************************************
         S T A N D A R D  R E Q U E S T
    *********************************************/ 
    if((Data[0] & 0x60) == 0x00){
     
      /*****************************************
      Standart Request: SET_ADRESS (5)
      ******************************************/ 
      if (Data[1] == SET_ADRESS){   // SET_Address
         UDADDR = Data[2];          // Set adress
         UEINTX &= 0xFE;            // Acknowledge
         while(!(UEINTX & 0x01));   // Wait for handshake
         UDADDR |= 0x80;            // Enable adress
      
      /*****************************************
      Standart Request: GET_DESCRIPTOR (6) 
      ******************************************/ 
      }else if(Data[1] == GET_DESCRIPTOR){       // GET_Descriptor
         
         
         if (Data[3] == DEVICE_DESCRIPTOR){
            
            SendDescriptor(DEV_Desc,0);
         
         }else if (Data[3] == CONFIGURATION_DESCRIPTOR){
         
            if (Data[6] == 9 & Data[7] == 0){
                SendDescriptor(CONF_Desc,0);
            }else{
                SendDescriptor(CONF_Desc,sizeof(CONF_Desc));
            }   
         
         }else if (Data[3] == STRING_DESCRIPTOR){
            
            // String Descriptor
            if(Data[2] == 0){
                SendDescriptor(DESCRIPTOR_StringLangID,0);
            
            }else if(Data[2] == 1){
                SendDescriptor(DESCRIPTOR_StringVendor,0);
            
            }else if(Data[2] == 2){
                SendDescriptor(DESCRIPTOR_StringProduct,0);
            
            }else if(Data[2] == 3){
                sprintf(Data,"ID:%u",deviceID);
                UEDATX = (2 + 2*strlen(Data));
                UEDATX = 0x03;            // Decsriptor type
                for(a=0;a<strlen(Data);a++){
                    UEDATX = Data[a];
                    UEDATX = 0;
                }     
                UEINTX &= 0xFE;            // Send data
                //SendDescriptor(DESCRIPTOR_StringSerial,0);
            }else{
                SendDescriptor(DESCRIPTOR_StringProduct,0);
                printf("No such string\r\n");
            }
            
         }else if (Data[3] == INTERFACE_DESCRIPTOR){
            // Interface
            UEINTX &= 0xFE;
            printf("Interface\r\n"); 
         }else if (Data[3] == ENDPOINT_DESCRIPTOR){
            // Endpoint
            UEINTX &= 0xFE;
            printf("EndPoint\r\n"); 
         }else if (Data[3] == DEVICE_QUALIFIER_DESCRIPTOR){
            // Device Qualifier
            UEINTX &= 0xFE;
            printf("QUAL_Desc\r\n"); 
         }else if (Data[3] == OTHER_SPEED_CONFIGURATION_DESCRIPTOR){
            // Speed Configuration
            UEINTX &= 0xFE;
            printf("Speed Configuration\r\n");
         }else if (Data[3] == 8){
            // Interface Power
            UEINTX &= 0xFE;
            printf("Interface Power\r\n");
         }else if (Data[3] == 34){
            // HID Report Desc
            UEINTX &= 0xFE;
            printf("HID_Rep\r\n");
         }else{
            // Anderer Descriptor
            UEINTX &= 0xFE;
            printf("Unknown descriptor\r\n");
         }
      
      /*****************************************
      Standart Request: SET_CONFIGURATION (9) 
      ******************************************/ 
      }else if(Data[1] == SET_CONFIGURATION){
         // SET_Configuration
         ConfigEndpoint(&CONF_Desc[EP1_INDEX]);
         ConfigEndpoint(&CONF_Desc[EP2_INDEX]);
         ConfigEndpoint(&CONF_Desc[EP3_INDEX]);
         
         ResetEndpoint(1);
         ResetEndpoint(2);
         ResetEndpoint(3);
         
         UEINTX &= 0xFE;            // Acknowledge
        
      /*****************************************
      Unhandeled Standard Request 
      ******************************************/ 
      }else{
         
         printf("Unknown Std Request: ");
         for(a=0;a<8;a++) printf("0x%X,",(unsigned int)Data[a]);
         printf("\r\n");
         UEINTX &= 0xFE;
      }
   

   /********************************************                                    
     C L A S S   S P E C I F I C  R E Q U E S T
   *********************************************/ 
   }else if ((Data[0] & 0x60) == 0x20){
               
    switch(Data[1]){
    
        case GET_LINE_CODING:
            for(a=0;a<Data[6];a++) UEDATX = LineCoding[a];  
            break;
        
        case SET_LINE_CODING:        
            while(UEBCLX < Data[6]); //Wait for arriving data
            for(a=0;a<Data[6];a++) LineCoding[a] = UEDATX; 
            break;
        
        case SET_CONTROL_LINE_STATE:
            //printf("SET_CONTROL_LINE_STATE\r\n");
            //Acknowledge, but ignore contents (unrelevant)
            break;
                
        default:            
            printf("Unknown SPECIFIC Request: ");
            for(a=0;a<8;a++) printf("0x%X,",(unsigned int)Data[a]);
            break;
        }
        
        UEINTX &= 0xFE;
        
   /********************************************
    U N K N O W N  R E Q U E S T
   *********************************************/
   }else{
            //Unknown Request
            printf("Unknown Request: ");
            for(a=0;a<8;a++) printf("0x%X,",(unsigned int)Data[a]);
            printf("\r\n");
            UEINTX &= 0xFE;
   }
}

void initUSB(void){

    unsigned char a;

    // Clear device registers
    UDCON = 0x01;   //First of all set the Detach bit
    USBCON = 0x20;  //..and the Freeze clock bit 
    OTGIEN = 0;     //Clear some Registers 
    UDIEN = 0;
    UDINT = 0;
    UHIEN = 0;
    UEIENX = 0;
    UPIENX = 0;
    UHWCON = 0x80;  //Set the Device Mode (ignore UID-pin) 
    //Clear the Endpoints 
    for(a=0;a<8;a++){
        UENUM = a;
        UEINTX = 0x00;
        UECONX = 0x00;
        UECFG1X = 0x00;
    }
    //Set up our device
    UHWCON = 0x81;  // Enable PAD
    USBCON = 0xB0;  // Enable Makro and OTG Pad
    for(a=0;a<3;a++) #asm("nop");  // delay
    USBCON = 0x90;  //Enable Clock 
    USBCON = 0xB1;  //Keep Freeze Clock + Enable Vtransition Interrupt
    UDIEN  = 0x18;  //Enable WakeUp and End of Reset Interrupt
    
    //Check if allready connected
    
    UDCON = 0;      //Attach the Device
    
    UsbTxCnt = 0;
    UsbRdCnt = 0;
    UsbBufCnt = 0;
         
}                   

void showDescriptor(char n){
    
    unsigned char par;
    
    UENUM = n;
    printf("%u ",UENUM);
    
    if(UECFG1X & 0x02){
    
        if(UECFG0X & 0x01) printf("  IN   ");
        else               printf("  OUT  ");
        
        par = (UECFG0X >> 6);
        
        if(par == 0x00)      printf("Control ");
        else if(par == 0x01) printf("Isocron ");
        else if(par == 0x02) printf("Bulk    ");
        else if(par == 0x03) printf("Interru ");
        
        par = (UECFG1X >> 4);
        printf("%u    ",(0x0008<<par));
  
        if(UEIENX == 0){
            printf("No enabled interrupts");
        }else{
            if(UEIENX & 0x01) printf("TXINE, ");
            if(UEIENX & 0x02) printf("STALLEDE, ");
            if(UEIENX & 0x04) printf("RXOUTE, ");
            if(UEIENX & 0x08) printf("RXSTPE, ");
            if(UEIENX & 0x10) printf("NAKOUTE, ");
            if(UEIENX & 0x40) printf("NAKINE, ");
            if(UEIENX & 0x80) printf("FLERRE, ");
        }
        
        printf(" %u  ",UEBCLX);
        
    }else{
        
        printf("Not configured");
    }
    
    printf("\r\n");
    UENUM = 0;
        
}


void printDescriptors(void){
       
    printf("*Endpoint Configuration*\r\n");
    printf("No. Dir: Type    Size: Interrupt(s):\r\n");
    showDescriptor(0);
    showDescriptor(1);
    showDescriptor(2);
    showDescriptor(3);
    
}

void printUSB(void){

    printf("*USB STATUS*\r\n");
    printf("Connected: ");
    if(USBSTA & 0x01) printf("YES\r\n");
    else              printf("NO\r\n");
    printf("Adressed:  ");
    if(UDADDR & 0x80) printf("YES (0x%X)\r\n",(0x7F & UDADDR));
    else              printf("NO\r\n");
    printDescriptors();
    
}

void initLineCoding(void){

    LineCoding[0] = 0x00;
    LineCoding[1] = 0x10;
    LineCoding[2] = 0x0E;
    LineCoding[3] = 0x00;   
    LineCoding[4] = 0x00;
    LineCoding[5] = 0x00;
    LineCoding[6] = 0x08;   
  
}

//void dumpLineCoding(void){
//
//    unsigned long bRate;
//    
//    printf("**LINE CODING**\r\n");
//    bRate = LineCoding[3];
//    bRate = (bRate << 8);
//    bRate += LineCoding[2];
//    bRate = (bRate << 8);
//    bRate += LineCoding[1];
//    bRate = (bRate << 8);
//    bRate += LineCoding[0];
//    bRate = bRate/1000;
//    printf("Baud Rate: %uKbps\r\n",bRate);
//    printf("Format:    ");
//    if(LineCoding[4] == 0) printf("1");
//    if(LineCoding[4] == 1) printf("1.5");
//    if(LineCoding[4] == 2) printf("2");
//    printf(" Stop bits\r\n");
//    printf("Parity:    ");
//    if(LineCoding[5] == 0) printf("None");
//    if(LineCoding[5] == 1) printf("Odd");
//    if(LineCoding[5] == 2) printf("Even");
//    if(LineCoding[5] == 3) printf("Mark");
//    if(LineCoding[5] == 4) printf("Space");
//    printf("\r\nData bits: %u\r\n",LineCoding[6]);
//    
//}



